<?php

/**
 * @file
 * PHP API kit for the Bits on the Run API
 *
 * Author:      Sergey Lashin
 * Copyright:   (c) 2009 Bits on the Run
 *
 * Version:     1.2
 * Updated:     Tue Jun 30 10:05:02 CEST 2009
 *
 * For the System API documentation see http://docs.bitsontherun.com/system-api
 */

class BotrAPI {
  protected $url = 'http://api.bitsontherun.com/v1';
  protected $library;

  protected $key, $secret;

  /**
   * Create the API interface.
   * Key and secret parameters can be found on the user's account page.
   */
  public function __construct($key, $secret) {
    $this->key = $key;
    $this->secret = $secret;

    // Determine which HTTP library to use:
    // check for cURL, else fall back to file_get_contents
    if (function_exists('curl_init')) {
      $this->library = 'curl';
    }
    else {
      $this->library = 'fopen';
    }
  }

  /**
   * RFC 3986 complient rawurlencode()
   * Only required for phpversion() <= 5.2.7RC1
   * See http://www.php.net/manual/en/function.rawurlencode.php#86506
   */
  protected function urlencode($input) {
    if (is_array($input)) {
      return array_map(array('_urlencode'), $input);
    }
    elseif (is_scalar($input)) {
      return str_replace('+', ' ', str_replace('%7E', '~', rawurlencode($input)));
    }
    else {
      return '';
    }
  }

  /**
   * Sign API call arguments.
   */
  protected function sign($args) {
    ksort($args);
    $sbs = NULL;
    foreach ($args as $key => $value) {
      if ($sbs != '') {
        $sbs .= '&';
      }
      // Construct Signature Base String.
      $sbs .= $key . '=' . strval($value);
    }

    // Add shared secret to the Signature Base String and generate the signature.
    $signature = sha1($sbs . $this->secret);

    return $signature;
  }

  /**
   * Add required api_* arguments.
   */
  protected function args($args) {
    $args['api_nonce'] = str_pad(mt_rand(0, 99999999), 8, STR_PAD_LEFT);
    $args['api_timestamp'] = time();

    $args['api_key'] = $this->key;

    if (!array_key_exists('api_format', $args)) {
      // Use the serialised PHP format, otherwise use format specified in the
      // call() args.
      $args['api_format'] = 'php';
    }

    // Urlencode array values.
    foreach ($args as $key => $value) {
      $args[$key] = $this->urlencode($value);
    }

    // Sign the array of arguments.
    $args['api_signature'] = $this->sign($args);

    return $args;
  }

  /**
   * Perform an API call.
   * Parameter $call should contain the name of the call and $args should be an
   * associative array of the arguments for the call.
   */
  public function call($call, $args = array()) {
    // Drupal has changed arg_separator to '&amp;', so set explicitly.
    $url = $this->url . $call . '?' . http_build_query($this->args($args), '', '&');

    $response = NULL;
    switch ($this->library) {
      case 'curl':
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($curl, CURLOPT_URL, $url);
        $response = curl_exec($curl);
        break;

      default:
        $response = file_get_contents($url);
    }

    return unserialize($response);
  }

  /**
   * Upload a file using the API.
   * The first parameter should be a returned link object and the second
   * parameter should be the path to the file to be uploaded.
   */
  public function upload($upload_link, $file_path, $api_format = 'php') {
    $url = $upload_link['protocol'] . '://' . $upload_link['address'] .
      $upload_link['path'] . '?key=' . $upload_link['query']['key'] .
      '&token=' . $upload_link['query']['token'] . '&api_format=' . $api_format;

    $post_data = array('file' => '@' . $file_path);
    $response = NULL;
    switch ($this->library) {
      case 'curl':
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $post_data);
        $response = curl_exec($curl);
        $err_no = curl_errno($curl);
        $err_msg = curl_error($curl);
        curl_close($curl);
        break;

      default:
        $response = 'Error: No cURL library';
    }

    if ($err_no == 0) {
      return unserialize($response);
    }
    else {
      return "Error #$err_no: $err_msg";
    }
  }
}
